﻿local addon	= PriorityList
local addonName, addonTable = ...
local L = addonTable.L
local AceGUI = LibStub("AceGUI-3.0")
local CHAR_ID = 999999
local tt2 = CreateFrame("GameTooltip", "PL_secondary_tooltip", nil, "GameTooltipTemplate")
tt2:SetScale(0.8)
tt2:Hide()

local function ShowTooltip(owner)
 GameTooltip:SetOwner(owner, "ANCHOR_NONE")
 GameTooltip:SetPoint("LEFT", owner, "RIGHT")
 GameTooltip:ClearLines()
 tt2:SetOwner(UIParent, "ANCHOR_NONE")
 tt2:SetPoint("RIGHT",GameTooltip,"LEFT")
 if owner.itemLink:GetText() ~= "nolink" then
  GameTooltip:SetHyperlink(owner.itemLink:GetText())
  GameTooltip:AddLine(L["<addon name>"])
 elseif owner.flag:GetText() == "msg" and table.getn(owner.bossdata) > 1 then
  addon:setbosstooltip(tt2, owner.bossdata, GameTooltip)
  tt2:Show()
 elseif owner.flag:GetText() ~= "char" and owner.flag:GetText() ~= "msg" then
  GameTooltip:AddLine(L["Itemnotfound"].." :(")
 end
 if owner.flag:GetText() ~= "msg" then
  GameTooltip:AddLine("|cff00ccff"..L["rightclicktoopen"].."|r")
 end
 GameTooltip:Show()
end

local function HideTooltip()
 GameTooltip:Hide()
 tt2:Hide()
end

local methods = {
  ["OnAcquire"] = function(self)
  self:SetFullWidth(true)
  self:SetHeight(30)
 end,
  ["SetTitle"] = function(self, title)
  self.title:SetText(title)
 end,
  ["SetDescription"] = function(self, desc)
  self.frame.description = desc
  self.description:SetText(desc)
 end,
  ["SetLink"] = function(self, link)
  self.itemLink:SetText(link)
 end,
  ["SetBoss"] = function(self, boss, bossicon)
  self.boss:SetText(boss)
  self.bossicon:SetTexture(bossicon)
  self.bossicon:Show()
 end,
  ["SetClick"] = function(self, func)
  self.frame:SetScript("OnClick", func)
 end,
  ["SetIcon"] = function(self, icon)
  self.icon:SetTexture(icon)
  self.icon:Show()
 end,
  ["OnRelease"] = function(self)
  self.icon:Hide()
  self.bossicon:Hide()
 end
}

local function Constructor()
    local name = "PriorityList_ListItem"..AceGUI:GetNextWidgetNum("PriorityList_ListItem");
    local button = CreateFrame("BUTTON", name, UIParent, "OptionsListButtonTemplate");
    button:SetHeight(30)
    button:SetWidth(1000)

    local background = button:CreateTexture(nil, "BACKGROUND")
    button.background = background
    background:SetTexture("Interface\\BUTTONS\\UI-Listbox-Highlight2.blp")
    background:SetBlendMode("ADD")
    background:SetVertexColor(0.5, 0.5, 0.5, 0.25)
    background:SetAllPoints(button)

    local icon = button:CreateTexture(nil, "OVERLAY")
    button.icon = icon
    icon:SetWidth(25)
    icon:SetHeight(25)
    icon:SetPoint("LEFT", button, "LEFT")

    local title = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    button.title = title
    title:SetHeight(7)
    title:SetJustifyH("LEFT")
    title:SetPoint("TOP", button, "TOP", 0, -6)
    title:SetPoint("LEFT", icon, "RIGHT", 2, 0)
    title:SetPoint("RIGHT", button, "RIGHT")

    local description = button:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
    button.description = description
    description:SetHeight(7)
    description:SetJustifyH("LEFT")
    description:SetPoint("BOTTOM", button, "BOTTOM", 0, 3)
    description:SetPoint("LEFT", icon, "RIGHT", 2, 0)
    description:SetPoint("RIGHT", button, "RIGHT")

    local bossicon = button:CreateTexture(nil, "OVERLAY")
    button.bossicon = bossicon
    bossicon:SetWidth(50)
    bossicon:SetHeight(30)
    bossicon:SetPoint("RIGHT", button, "RIGHT", 0, 0)

    local boss = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    button.boss = boss
    boss:SetHeight(7)
    boss:SetJustifyH("RIGHT")
    boss:SetPoint("TOP", button, "TOP",0,-8)
    boss:SetPoint("LEFT", button, "LEFT")
    boss:SetPoint("RIGHT", bossicon, "LEFT")

    local itemLink = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    button.itemLink = itemLink
    local flag = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    button.flag = flag

    button:SetScript("OnEnter", function() ShowTooltip(button) end)
    button:SetScript("OnLeave", function() HideTooltip() end)

    local needers = {}
    local bossdata = {}
    button.bossdata = bossdata

    local widget = {
        frame = button,
        title = title,
        icon = icon,
        description = description,
        background = background,
        itemLink = itemLink,
        boss = boss,
        bossicon = bossicon,
        flag = flag,
        bossdata = bossdata,
        type = "PriorityList_ListItem"
    }
    for method, func in pairs(methods) do
        widget[method] = func
    end

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType("PriorityList_ListItem", Constructor, 20)