﻿local addonName, addonTable = ...
local L = addonTable.L
local addon = PriorityList
local AceGUI = LibStub("AceGUI-3.0")

local low = string.lower

-- string colors
local WARLOCK_PURPLE = "|cff9482C9"
local MONK_GREEN  = "|cff00FF96"
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local WHITE       = "|cffffffff"
local CHOCOLATE   = "|cffCD661D"
local function AddColor(str,color)
 return color..str.."|r"
end

addon.opt = {}

function addon:IsLooted(itemID)
 if itemID < 0 then 
  return true
 else
  return false
 end
end

function addon:SetLooted(char,itemID,flag,refresh)
local pos = addon:GetItemPos(char,itemID)
 if PLWish_List[char] then
  if flag == false then
   PLWish_List[char][pos] = abs(PLWish_List[char][pos])
  else
   PLWish_List[char][pos] = -abs(PLWish_List[char][pos])
  end
  if refresh then
   addon:GenerateItemsList("main")
   addon:PopulateItemsList_new()
  end
 end
end

function addon:GetItemPos(name,itemID)
local result = table.getn(PLWish_List[name])
 for index,ID in pairs(PLWish_List[name]) do
  if ID == itemID or ID == -itemID then result = index end
 end
 return result
end

function addon:AddItem(name,itemID,refresh,position)
if table.getn(addon.import_var.Wish_List) == 0 then
 if not PLWish_List[name] then
  PLWish_List[name] = {}
 end
 if PLWish_List[name] then
  if not addon:TableMatch(PLWish_List[name],itemID,true) then
   if position and position <= table.getn(PLWish_List[name]) then
    table.insert(PLWish_List[name],position,itemID)
   else
    table.insert(PLWish_List[name],itemID)
   end
   addon:SetStatus(string.format(L["st_additem"],itemID,name)) --sysmess
  end
  if refresh then
   addon:GenerateItemsList("main")
   addon:PopulateItemsList_new()
  end
 end
else
 addon:SetStatus(L["listdisabled"],smc_r,smc_g,smc_b)
end
end

function addon:DeleteItem(itemID, refresh, charonly, name)
 local tmp_ID = nil
 local delitem = function()
  if charonly then
   if PLWish_List[name] then
    for i,ID in pairs(PLWish_List[name]) do
     if ID == itemID then
      tmp_ID = i
      break
     end
    end
   if tmp_ID ~= nil then
    table.remove(PLWish_List[name],tmp_ID)
   end
  end
 else
  table.foreach(PLWish_List,
   function(k,v) --foreach
    for i,ID in pairs(v) do
     if ID == itemID or ID == -itemID then
      tmp_ID = i 
      break
     end
    end
    if tmp_ID ~= nil then
     table.remove(v,tmp_ID) 
    end
   end)
 end
 if refresh then
  addon:GenerateItemsList("main")
  addon:PopulateItemsList_new()
 end
 end
 -- if it doesn't refresh the list, it's a way to change an itemid and it doesn't require confirms
 if refresh then addon:ConfirmCheck(delitem) else delitem() end
end

function addon:PopulateItemsList_new()
local LISTEMPTY_ITEM = {itemID = 999997,
                        name_col = L["listempty"],
                        link = "nolink",
                        flag = "msg",
                        spacing = 0}

 local list = ""
 if addon.opt.tab_selected == L["itemlist"] then -- items list tab
  list = "items"
 elseif addon.opt.tab_selected == L["opt_bosslist"] then -- items list tab
  list = "bosslist"
 elseif addon.opt.tab_selected == L["opt_Import/Export"] then -- import list tab
  list = "import"
 elseif addon.opt.tab_selected == L["opt_wish"] then -- wish list tab
  list = "wishlist"
 end
 if list ~= "" then
  if addon.opt.frm_items then
   addon.opt.frm_items:ReleaseChildren()
  end
  local found = false
  if addon:SortItemsList(addon.opt.edt_search:GetText(),list) then
   found = true
  end
  if not found then --list empty
   addon:NewButton(LISTEMPTY_ITEM,list)
  end
 end
end

-- items list tab creation
function addon:TabInit_ItemsList(container)
if container then

-- buttons panel
 addon.opt.upperpanel = AceGUI:Create("SimpleGroup")
 addon.opt.upperpanel:SetFullWidth(true)
 addon.opt.upperpanel:SetLayout("Flow")
 container:AddChild(addon.opt.upperpanel)

 addon.opt.edt_search:Show()
 addon.opt.edt_search_clear:Show()
 addon.opt.frm_items = AceGUI:Create("ScrollFrame")
 addon.opt.frm_items:SetFullWidth(true)
 addon.opt.frm_items:SetFullHeight(true)
 addon.opt.frm_items:SetLayout("List")
 container:AddChild(addon.opt.frm_items)

 addon:PopulateItemsList_new()
end
end
-- items list tab creation end

--
function addon:SortItemsList(filter,list)
local CHAR_ID   = 999999
local IMPORT_ID = 999998
local BOSS_ID   = 999995
local BOSS_SPACER_ID = 999996
local LISTEMPTY_ID = 999997
local result = false
local ITEMS_NOT_IN_SM = {itemID = LISTEMPTY_ID,
                        name_col = L["nolistinsimplemode"],
                        link = "nolink",
                        flag = "msg",
                        spacing = 0}


 --indexed foreach
 local function foreachinorder(t,f)
  local keys = {}
  for k,_ in pairs(t) do
    keys[#keys+1] = k
  end
  table.sort(keys,function (a,b)
   return low(a) < low(b)
  end)
  for _,k in ipairs(keys) do
   f(k,t[k])
  end
 end
--

 local function InsertItem(k,v,ischild,spacing,flag,sender)
  if list == "import" and k == 0 then --it's a "new item"
   -- do nothing
  else
   local found_name = false
   local found_need = false
   local needers,col_needers = addon:MergeNames(addon.lootlist[addon:getitemsraid(k)][k])
   local itemName,itemLink,itemTexture = addon:Extr_ItemProp(k)
   local col_name = itemName
   local ID = k
   local looted = false
   if ID < 0 then
    ID = abs(ID)
    looted = true
   end
   
   -- raid selection v2.0.6
   local passes = false
   local bossname,bossicon,raidname = addon:MatchBossDrop(ID)
   if pl_options["current_raid"] == raidname or pl_options["current_raid"] == L["all_raids"] then
    passes = true
   end
   if list == "import" then passes = true end -- no matter which raid is selected while importing

   if passes then
   if col_name:lower():find(filter) then found_name = true end
   if needers:lower():find(filter) then found_need = true end
   if found_name or found_need or list ~= "items" then
    if found_name and filter ~= "" and filter ~= " " then
     col_name = string.gsub(col_name:lower(),filter,AddColor(filter,LIGHT_RED)..GOLD)
    end
    col_name = AddColor(col_name,GOLD)
    if not addon:IsSimpleMode() then
     if found_need and filter ~= "" and filter ~= " " then
      col_needers = string.gsub(col_needers:lower(),filter,AddColor(filter,LIGHT_RED)..GREY)
     end
     col_needers = AddColor(col_needers,GREY) --needers color also in MergeNames()
    else
     col_needers = ""
    end

    if flag == "add" then
     col_name = AddColor(L["New:"].." ", LIGHT_GREEN)..col_name
    elseif flag == "del" then
     col_name = AddColor(L["Will be deleted:"].." ", LIGHT_RED)..col_name
    elseif flag and flag:find("mod") then --mod flags in the same string
     col_name = AddColor(L["Changed:"].." ", LIGHT_BLUE)..col_name
    end
    addon:NewButton({itemID = ID,
                                    looted = looted,
                                    name = itemName,
                                    name_col = col_name,
                                    link = itemLink,
                                    texture = itemTexture,
                                    needs = needers,
                                    needs_col = col_needers,
                                    bossname = bossname,
                                    bossicon = bossicon,
                                    flag = flag,
                                    child = ischild,
                                    spacing = spacing,
                                    sender = sender},list)
   else
  --
   end
   end -- raid selection v2.0.6
  end
 end
 
  local function getcharitemscount(tab)
   local result = 0
   local raid = ""
   if tab then
   table.foreach(tab,function(k,v)
    if list == "import" or addon:itemisinraid(v,pl_options["current_raid"]) then
     result = result+1
    end
   end)
   end
   return result
  end

 local function InsertChar(name,items,sender)
  local child   = ""
  local spacing = 0
  local opened = true
  local snd
  local counter_str,counter
  
   if name:lower():find(filter) then
    --insert char
    if list == "import" then
     child = sender
     spacing = 20
    end

    if list == "import" then
     opened = addon.import_var.folding[sender][name]
    elseif list == "items" then
     opened = addon.items_folding[name]
    elseif list == "wishlist" then
     opened = addon.wl_folding[name]
    end
    if sender == nil then snd = "" else snd = sender end

    counter = getcharitemscount(items)
    if counter == 0 then
     counter_str = AddColor(tostring(counter).." "..L["itemsinlist"],LIGHT_RED)
    else
     counter_str = AddColor(tostring(counter).." "..L["itemsinlist"],LIGHT_GREEN)
    end

    if counter < 1 and list == "items" then return end
    
    addon:NewButton({itemID = CHAR_ID,
                                    name = name,
                                    name_col = AddColor(name,GREEN),
                                    link = "nolink",
                                    needs_col = counter_str,
                                    flag = list == "items" and "msg" or "char",
                                    child = child,
                                    opened = opened,
                                    sender = snd,
                                    spacing = spacing},list)
    --insert char's items
    if list == "import" then
     local exists = false
     if opened then
     table.foreach(items,
      function(k,v)
       if PLWish_List[name] ~= nil then
        exists = false
        table.foreach(PLWish_List[name], function(index,id)
         local changed = false
         local cng = "mod"
         if id == v or id == -v then --already exists
          if (addon:IsLooted(v) and not addon:IsLooted(id)) then
           changed = true
           cng = cng..AddColor("("..L["Lootedflag"].." "..AddColor(L["added"],LIGHT_GREEN),WHITE)..") "
          elseif (addon:IsLooted(id) and not addon:IsLooted(v)) then
           changed = true
           cng = cng..AddColor("("..L["Lootedflag"].." "..AddColor(L["removed"],LIGHT_RED),WHITE)..") "
          end
          if (k > index) then
           changed = true
           cng = cng..AddColor(" -"..k-index.." ",LIGHT_RED)
          elseif (k < index) then
           changed = true
           cng = cng..AddColor(" +"..index-k.." ",LIGHT_GREEN)
          end
          if changed then
           InsertItem(v,addon.import_var.Items_List[v][sender],name,40,cng,sender)
          else
           InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"",sender)
          end
          exists = true
         end
        end)
        else -- char doesn't exist
         InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"add",sender)
         exists = true
        end
        if not exists then
         InsertItem(v,addon.import_var.Items_List[v][sender],name,40,"add",sender)
        end
      end)
      end -- opened
    else -- list == wishlist or bosslist
    if opened or addon:IsSimpleMode() then
     table.foreach(items,
     function(k,v)
      InsertItem(v,Items_List[v],name,20,"")
     end)
    end
   end
   end
  end

 local function InsertPlayer(name,items)
                                  addon:NewButton({itemID = IMPORT_ID,
                                  name = name,
                                  name_col = AddColor(name,LIGHT_BLUE),
                                  link = "nolink",
                                  flag = "char",
                                  spacing = 0},list)
 end
 
 local function InsertBoss(name,items)
  local needers = ""
  local col_needers = ""
  local tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
  -- raid selection v2.0.6
  if pl_options["current_raid"] == name or pl_options["current_raid"] == L["all_raids"] then
   if addon:bl_bossopened(name) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
   addon:NewButton({itemID = BOSS_ID,
                    name = name,
                    name_col = AddColor(name,CHOCOLATE),
                    link = "nolink",
                    flag = "msg",
                    bossdata = false,
                    icon = tmp_icon,
                    spacing = 0},list)
   foreachinorder(items,function(bossname,j)
    tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
    if addon:bl_bossopened(name) then
    if addon:bl_bossopened(name,bossname) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
    local b_icon = ""
    if j["icon"] then b_icon = j["icon"] end
     addon:NewButton({itemID = BOSS_ID,
                    name = bossname,
                    name_col = AddColor(bossname,CHOCOLATE),
                    bossicon = b_icon,
                    icon = tmp_icon,
                    link = "nolink",
                    bossdata = true,
                    raid = name,
                    flag = "msg",
                    spacing = 10},list)
   for diff,tab in pairs(j) do
    if diff ~= "icon" then
     tmp_icon = "Interface\\Common\\Indicator-Gray.blp"
    if addon:bl_bossopened(name,bossname) then
     if addon:bl_bossopened(name,bossname,diff) then tmp_icon = "Interface\\Common\\Indicator-Green.blp" end
        addon:NewButton({itemID = BOSS_ID,
                     name = diff,
                     name_col = AddColor(diff,LIGHT_RED),
                     icon = tmp_icon,
                     link = "nolink",
                     raid = name,
                     bossdata = false,
                     boss = bossname,
                     flag = "msg",
                     spacing = 20},list)
    if addon:bl_bossopened(name,bossname,diff) then
    table.foreach(tab,function(k,v)
      local itemName,itemLink,itemTexture = addon:Extr_ItemProp(v)
     
     local needers = ""
     local needers_col = "" 
     if not addon:IsSimpleMode() then
      --if Items_List[v] then needers,needers_col = addon:MergeNames(Items_List[v]) end
      if addon.lootlist[addon:getitemsraid(v)][v] then needers,needers_col = addon:MergeNames(addon.lootlist[addon:getitemsraid(v)][v]) end
     else
      local p = UnitName("player")
      if addon:TableMatch(PLWish_List[p],v,true) then needers_col = AddColor(p,GREY) end
     end
      if v == -1 then
      addon:NewButton({itemID = BOSS_SPACER_ID,
                       looted = false,
                       name = "",
                       name_col = L["boss_spacer"],
                       link = "nolink",
                       texture = "",
                       spacing = 30,
                       flag   = "msg",
                       sender = ""},list)
     else
      local statcolor = GREY;
      local suff = ""
      local _, _, _, _, _, class, subclass, _, slot = GetItemInfo(v)
      
      if itemLink then
       local stats = GetItemStats(itemLink)
       for stat, value in pairs(stats) do
        if stat == "ITEM_MOD_INTELLECT_SHORT" then
         statcolor = WARLOCK_PURPLE
         suff = L["INT"]
         --break -- do not break here because spirit is always below intellect
    elseif stat == "ITEM_MOD_AGILITY_SHORT" then
         statcolor = YELLOW
         suff = L["AGI"]
         break
    elseif stat == "ITEM_MOD_STRENGTH_SHORT" then
         statcolor = LIGHT_RED
         suff = L["STR"]
         break
   elseif stat == "ITEM_MOD_SPIRIT_SHORT" then
         statcolor = MONK_GREEN
          suff = L["SPI"]
         break
        end
       end
       end  

      local nc = AddColor(itemName,GOLD)

     if _G[slot] then
      nc = AddColor(suff,statcolor).." "..nc..AddColor(" ("..subclass.." ".._G[slot]..")",WHITE)
     end

     addon:NewButton({itemID = v,
                      looted = false,
                      name = itemName,
                      name_col = nc,
                      link = itemLink,
                      texture = itemTexture,
                      spacing = 30,
                      sender = AddColor(needers_col,GREY)},list)
    end
   end)
 end end end --bossopened
  end 
  end -- for do
  end)-- foreachinorder
 end
 end -- raid selection v2.0.6

 filter = filter:lower()
 local mainlist = {}
 if    list == "items" then mainlist = Items_List
elseif list == "bosslist" then mainlist = PriorityList.boss_drops_table
elseif list == "import" then mainlist = addon.import_var.Wish_List
elseif list == "wishlist" then mainlist = PLWish_List
 end
 if list == "wishlist" then
  if addon:IsSimpleMode() then
   local p = UnitName("player")
   if mainlist[p] then 
    if not result then result = true end
    InsertChar(p,mainlist[p])
   end
  else--officer mode
  foreachinorder(mainlist,
   function(k,v)
    -- wishlist 1 layer folding
     if addon.wl_folding[k] == nil then addon.wl_folding[k] = false end
    --
    if not result then result = true end
    if type(k) == "string" then
      InsertChar(k,v)
    elseif type(k) == "number" then
     InsertItem(k,v)
    end
   end)
  end --simple mode check
 elseif list == "bosslist" then
  foreachinorder(mainlist,
   function(k,v)
    if not result then result = true end
    InsertBoss(k,v)
   end)
 elseif list == "items" then
  if addon:IsSimpleMode() then
   if not result then result = true end
   addon:NewButton(ITEMS_NOT_IN_SM,list)
  else--officer mode
  -- items list folding v2.0.9
  local flist = {}
  table.foreach(mainlist,
   function(k,v)
    if type(k) == "number" then
     if not result then result = true end
     local boss = addon:MatchBossDrop(k)
     if not addon.items_folding[boss] then addon.items_folding[boss] = false end
     if not flist[boss] then flist[boss] = {} end
     table.insert(flist[boss], k)
    end
   end)
   -- I use InsertChar as a little hack
   foreachinorder(flist, function(k,v) InsertChar(k,v) end)
  end
 else -- import list
  table.foreach(mainlist,function(k,v)
   if not result then result = true end
   InsertPlayer(k,v)
   foreachinorder(v,function(i,j)
    -- import list folding
     if addon.import_var.folding[k] == nil then addon.import_var.folding[k] = {} end
     if addon.import_var.folding[k][i] == nil then addon.import_var.folding[k][i] = false end
    --
    InsertChar(i,j,k)
    -- pending deletion items
    if PLWish_List[i] then
    if addon.import_var.folding[k][i] then
    table.foreach(PLWish_List[i],
     function(index,itemID)
      if addon.import_var.Deleting_List[k] and addon.import_var.Deleting_List[k][i] then
       if addon:TableMatch(addon.import_var.Deleting_List[k][i],itemID,true) then
        InsertItem(itemID,Items_List[itemID],name,40,"del",k)
       end
      end
     end)
     end
     end
    end)
  end)
 end--lists

 return result
end